/*
 * odometrie.c
 *
 *  Created on: 07.10.2010
 *      Author: Stephan
 */
#include "asuro.h"
#include <util/delay.h>
#include <math.h>

#define RADDURCHMESSER 40.0//40.0
#define RADABSTAND 104.0//104.0
#define K1 32
#define K2 16
#define K3 8
#define K4 4
#define K5 2
#define K6 1
#define LINKS 1
#define RECHTS 2

uint16_t h_left = 0;
uint16_t l_left = 1000;
uint16_t h_right = 0;
uint16_t l_right = 1000;

void int_to_string(unsigned int zahl, char* text) {
	int8_t i;
	for (i = 2; i >= 0; i--) {
		text[i] = (zahl % 10) + '0';
		zahl = zahl / 10;
	}
	if (text[0] == '0') {
		text[0] = ' ';
		if (text[1] == '0')
			text[1] = ' ';
	}
}

uint16_t odometrie_left(void) {
	uint16_t data[2];
	OdometrieData(data);
	return data[0];
}

uint16_t odometrie_right(void) {
	uint16_t data[2];
	OdometrieData(data);
	return data[1];
}

void hl_odometrie(void) {
	uint16_t data[2];
	MotorDir(FWD, FWD);
	MotorSpeed(255, 255);
	uint16_t i;
	for (i = 0; i < 2000; i++) {
		OdometrieData(data);
		if (h_left < data[0]) {
			h_left = data[0];
		}
		if (l_left > data[0]) {
			l_left = data[0];
		}
		if (h_right < data[1]) {
			h_right = data[1];
		}
		if (l_right > data[1]) {
			l_right = data[1];
		}
	}
	MotorDir(BREAK, BREAK);
}

void drive(float strecke,uint8_t speed) {
	uint16_t wechsel_l = 0, wechsel_r = 0;
	uint16_t pos_l, pos_r;
	uint16_t grenze_r, grenze_l;
	MotorDir(FWD, FWD);
	MotorSpeed(speed-6, speed);
	pos_l = odometrie_left();
	pos_r = odometrie_right();
	while ((wechsel_l / 12.0 / 5.0 * M_PI * RADDURCHMESSER <= strecke) && (wechsel_r / 12.0 / 5.0 * M_PI * RADDURCHMESSER <= strecke)) {
		if (h_right < odometrie_right()) {
			h_right = odometrie_right();
		}
		if (l_right > odometrie_right()) {
			l_right = odometrie_right();
		}
		grenze_r = (h_right + l_right) / 2;
		if (h_left < odometrie_left()) {
			h_left = odometrie_left();
		}
		if (l_left > odometrie_left()) {
			l_left = odometrie_left();
		}
		grenze_l = (h_left + l_left) / 2;
		//Die Wechsel rechts zhlen
		if (odometrie_right() >= grenze_r && pos_r <= grenze_r) {
			wechsel_r++;
			pos_r = odometrie_right();
		}
		if (odometrie_right() <= grenze_r && pos_r >= grenze_r) {
			wechsel_r++;
			pos_r = odometrie_right();
		}
		//Die Wechsel links zhlen
		if (odometrie_left() >= grenze_l && pos_l <= grenze_l) {
			wechsel_l++;
			pos_l = odometrie_left();
		}
		if (odometrie_left() <= grenze_l && pos_l >= grenze_l) {
			wechsel_l++;
			pos_l = odometrie_left();
		}
	}
	char text[4];
	int_to_string(wechsel_l, text);
	SerWrite("l: ", 3);
	SerWrite(text, 4);
	int_to_string(wechsel_r, text);
	SerWrite("   r: ", 6);
	SerWrite(text, 4);
	MotorDir(BREAK, BREAK);
}

uint8_t umdrehung(float umdrehungen) {
	uint16_t wechsel = 0;
	uint16_t pos;
	uint16_t grenze;
	if (umdrehungen > 0) {
		MotorDir(BREAK, FWD);
		MotorSpeed(0, 150);
		grenze = (h_right + l_right) / 2;
		pos = odometrie_right();
		while (wechsel / 12.0 / 5.0 <= umdrehungen) {
			//Odometriewerte rechts korrigieren
			if (h_right < odometrie_right()) {
				h_right = odometrie_right();
			}
			if (l_right > odometrie_right()) {
				l_right = odometrie_right();
			}
			grenze = (h_right + l_right) / 2;
			//Die Wechsel zhlen
			if (odometrie_right() >= grenze && pos <= grenze) {
				wechsel++;
				pos = odometrie_right();
			}
			if (odometrie_right() <= grenze && pos >= grenze) {
				wechsel++;
				pos = odometrie_right();
			}
		}
	} else {
		MotorDir(FWD, BREAK);
		MotorSpeed(150, 0);
		grenze = (h_left + l_left) / 2;
		pos = odometrie_left();
		while (wechsel / 12.0 / 5.0 <= umdrehungen * -1) {
			//Odometriewerte links korrigieren
			if (h_left < odometrie_left()) {
				h_left = odometrie_left();
			}
			if (l_left > odometrie_left()) {
				l_left = odometrie_left();
			}
			grenze = (h_left + l_left) / 2;
			//Die Wechsel zhlen
			if (odometrie_left() >= grenze && pos <= grenze) {
				wechsel++;
				pos = odometrie_left();
			}
			if (odometrie_left() <= grenze && pos >= grenze) {
				wechsel++;
				pos = odometrie_left();
			}

		}
	}
	MotorDir(BREAK, BREAK);
	return 1;
}

uint8_t winkel_fahren(int16_t winkel) {
	return umdrehung((winkel * RADABSTAND * 2.0) / (360.0 * RADDURCHMESSER));
}

void Init2(void) {
	TCCR1A = (1 << WGM11) | (1 << WGM10) | (1 << COM1A1);
	//TCCR1A = (1 << COM1A1) | (1 << WGM11);
	TCCR1B = (1 << WGM13) | (1 << WGM12) | (1 << CS10);
	ICR1 = 0xFFFF;
}

int main(void) {
	Init();
	char text[11];
	uint8_t sicherung;
	while (1) {
		sicherung = 0;
		while (PollSwitch() != sicherung) {
			sicherung = PollSwitch();
		}
		switch (sicherung) {
		case K1:
			hl_odometrie();
			break;
		case K2:
			//hl_linedata();
			break;
		case K3:
			_delay_ms(500);
			drive(300,255);
			/*SerWrite("Links: ", 7);
			 SerWrite(" H:", 3);
			 int_to_string(h_left, text);
			 SerWrite(text, 3);
			 SerWrite(" L:", 3);
			 int_to_string(l_left, text);
			 SerWrite(text, 3);
			 SerWrite("    Rechts: ", 11);
			 SerWrite(" H:", 3);
			 int_to_string(h_right, text);
			 SerWrite(text, 3);
			 SerWrite(" L:", 3);
			 int_to_string(l_right, text);
			 SerWrite(text, 3);
			 _delay_ms(1000);*/
			break;
		case K4:
			winkel_fahren(90);
			break;
		case K5:
			FrontLED(ON);
			_delay_us(100);
			FrontLED(OFF);
			break;
		case K6:
			winkel_fahren(-90);
			break;
		}
	}
}
